package gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileReader;
import java.util.ResourceBundle;

public class AdminsDialog
        extends JDialog implements ActionListener {
  
  private JButton cancelButton, okButton;
  private JComboBox rulesComboBox, gameComboBox;
  private JTextField admin1Field, admin2Field, prize1Field, prize2Field;
  private JLabel admin1Label, admin2Label, prize1Label, prize2Label, rulesLabel, gameLabel;
  
  private Dimension labelDim = new Dimension();
  
  private boolean accepted = false;
  
  public AdminsDialog(Frame frame) {
    super(frame, true);
    
    ResourceBundle bundle = ResourceBundle.getBundle("GUI");
    
    cancelButton = new JButton(bundle.getString("cancel"));
    okButton = new JButton(bundle.getString("ok"));
    
    gameComboBox = new JComboBox(getGames());
    rulesComboBox = new JComboBox(getRules());
    
    admin1Field = new JTextField();
    admin2Field = new JTextField();
    prize1Field = new JTextField("20 VIP");
    prize2Field = new JTextField("10 VIP");
    
    admin1Label = new JLabel(bundle.getString("admin1") + ": ");
    admin2Label = new JLabel(bundle.getString("admin2") + ": ");
    gameLabel = new JLabel(bundle.getString("gameComboBox") + ": ");
    prize1Label = new JLabel(bundle.getString("goldPrize") + ": ");
    prize2Label = new JLabel(bundle.getString("silverPrize") + ": ");
    rulesLabel = new JLabel(bundle.getString("rulesComboBox") + ": ");
    
    adjustLabelSize();
    
    JPanel buttonPanel = new JPanel();
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);
    buttonPanel.setBorder(BorderFactory.createEtchedBorder());
    
    JPanel centerPanel = new JPanel();
    centerPanel.setLayout(new BoxLayout(centerPanel, BoxLayout.Y_AXIS));
    centerPanel.add(makeRow(gameLabel, gameComboBox));
    centerPanel.add(makeRow(rulesLabel, rulesComboBox));
    centerPanel.add(Box.createVerticalStrut(10));
    centerPanel.add(makeRow(admin1Label, admin1Field));
    centerPanel.add(makeRow(admin2Label, admin2Field));
    centerPanel.add(makeRow(prize1Label, prize1Field));
    centerPanel.add(makeRow(prize2Label, prize2Field));
    
    setLayout(new BorderLayout());
    add(centerPanel, BorderLayout.NORTH);
    add(buttonPanel, BorderLayout.SOUTH);
    
    cancelButton.addActionListener(this);
    okButton.addActionListener(this);
    gameComboBox.addActionListener(this);
    
    setTitle(bundle.getString("adminsDialogTitle"));
    setSize(400, 220);
    setLocationRelativeTo(null);
  }
  
  public void actionPerformed(ActionEvent e) {
    if (e.getSource() == okButton) {
      accepted = true;
      setVisible(false);
    } else if (e.getSource() == cancelButton) {
      accepted = false;
      setVisible(false);
    } else if(e.getSource() == gameComboBox) {
      rulesComboBox.removeAllItems();
      rulesComboBox.validate();
      for(String name : getRules()) {
        rulesComboBox.addItem(name);
      }
    }
  }
  
  private void adjustLabelSize() {
    labelDim = admin1Label.getPreferredSize().width > labelDim.width ?
      admin1Label.getPreferredSize() : labelDim;
    labelDim = admin2Label.getPreferredSize().width > labelDim.width ?
      admin2Label.getPreferredSize() : labelDim;
    labelDim = prize1Label.getPreferredSize().width > labelDim.width ?
      prize1Label.getPreferredSize() : labelDim;
    labelDim = prize2Label.getPreferredSize().width > labelDim.width ?
      prize2Label.getPreferredSize() : labelDim;
    labelDim = gameLabel.getPreferredSize().width > labelDim.width ?
      gameLabel.getPreferredSize() : labelDim;
    labelDim.width += 15;
  }
  
  public String getAdmin1() {
    return admin1Field.getText();
  }
  
  public String getAdmin2() {
    return admin2Field.getText();
  }
  
  public String getHomePage() {
    try {
      File file = new File("templates/" + gameComboBox.getSelectedItem().toString() + "/pokal-liste.txt");
      BufferedReader br = new BufferedReader(new FileReader(file));
      String line = br.readLine();
      br.close();
      return line;
    }catch(Exception exc) {
      return null;
    }
  }
  
  public String getPrize1() {
    return prize1Field.getText();
  }
  
  public String getPrize2() {
    return prize2Field.getText();
  }
  
  public File getRulesFile() {
    try {
      File file = new File("templates/" + gameComboBox.getSelectedItem().toString() + "/rules/" + rulesComboBox.getSelectedItem().toString());
      System.out.println(file);
      if(file.exists()) return file;
    }catch(Exception exc) { }
    return null;
  }
  
  private String[] getGames() {
    File file = new File("templates");
    
    File[] files = file.listFiles(new FileFilter() {
      public boolean accept(File file) {
        return file.isDirectory();
      }
    });
    if(files == null) return new String[] {""};
    
    String[] names = new String[files.length];
    for(int i = 0; i < files.length; i++) {
      names[i] = files[i].getName();
    }
    return names;
  }
  
  private String[] getRules() {
    try {
      File file = new File("templates/" + gameComboBox.getSelectedItem().toString() + "/rules");
      String[] names = file.list();
      if(names != null) return names;
    }catch(Exception exc) { }
    return new String[] { "" };
  }
  
  public File getThankFile() {
    try {
      File file = new File("templates/" + gameComboBox.getSelectedItem().toString() + "/thank.txt");
      if(file.exists()) return file;
    }catch(Exception exc) { }
    return null;
  }
  
  private JPanel makeRow(JLabel label, JComponent comp) {
    JPanel panel = new JPanel(new BorderLayout());
    label.setPreferredSize(labelDim);
    label.setHorizontalAlignment(JLabel.RIGHT);
    panel.add(label, BorderLayout.WEST);
    panel.add(comp, BorderLayout.CENTER);
    return panel;
  }
  
  public boolean showDialog() {
    accepted = false;
    setVisible(true);
    return accepted;
  }
  
}
