package gui;

import divisionizer.Main;
import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import autowrite.*;
import divisionizer.ListName;
import java.util.prefs.Preferences;
import javax.imageio.ImageIO;
import tournament.cup.Cup;
import tournament.Tournament;
import tournament.div.League;

public class HeadFrame
        extends JFrame implements ActionListener, KeyEventDispatcher {
  
  private AdminsDialog adminsDialog;
  private Autowriter autowriter;
  private BackgroundPanel backgroundPanel;
  private CoordinatePanel coordinatePanel;
  private JButton announceButton;
  private JLabel clipboardLabel;
  private KeyboardFocusManager manager;
  private PlayersDialog playersDialog;
  private ProgramMenuBar menuBar;
  private Tournament tournament = null;
  private JMenuItem fastAnnounceMenuItem, lightningAnnounceMenuItem;
  
  private ResourceBundle bundle = ResourceBundle.getBundle("GUI");
  private String gold = "Gold", silver = "Silver";
  
  public HeadFrame() {
        
    try {
      initComponents();
    }catch(Exception exc) {
      exc.printStackTrace();
    }
    
    update();
    
    setLocationRelativeTo(null);
    setVisible(true);
  }
  
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
    
    if (src == announceButton) {
      announce(3000);
    }else if(src == fastAnnounceMenuItem) {
      announce(800);
    } else if(src == lightningAnnounceMenuItem) {
      announce(50);
    }
  }
  
  private void announce(int delay) {
    if(!coordinatePanel.isLocked()) {
      coordinatePanel.flash();
      return;
    }
    
    autowriter.write(tournament.getUnannounced(true), delay);
    update();
  }
  
  public void closeProgram() {
    if (tournament != null && !tournament.isEmpty()) {
      int sel = JOptionPane.showConfirmDialog(this,
              bundle.getString("sureMessage"));
      
      if (sel != JOptionPane.YES_OPTION) {
        return;
      }
      
      if(tournament.isFinished() && Preferences.userNodeForPackage(divisionizer.Main.class).getBoolean("checkList", false)) {
        String url = adminsDialog.getHomePage();
        
        if(url != null) {
          ListName[] names = divisionizer.PokalListe.check(url);
          String gold = tournament.getGold().trim().toLowerCase();
          String silver = tournament.getSilver().trim().toLowerCase();
          
          String text = bundle.getString("checkPokalListe") + "\n\n";
          boolean show = false;
          
          for(ListName n : names) {
            if(n == null) continue;
            if(n.name.equals(gold)) text += n.name + ", " + (n.golds + 1) + ", " + n.silvers + '\n';
            else if(n.name.equals(silver)) text += n.name + ", " + n.golds + ", " + (n.silvers + 1) + '\n';
            else continue;
            show = true;
          }
          
          if(show) JOptionPane.showMessageDialog(this, text);
        }
      }
    }
    
    System.exit(0);
  }
  
  public boolean dispatchKeyEvent(KeyEvent e) {
    if (e.getKeyCode() == KeyEvent.VK_F1) {
      manager.redispatchEvent(coordinatePanel, e);
      return true;
    }
    return false;
  }
  
  private void initComponents() throws Exception {
    
    setDefaultCloseOperation(EXIT_ON_CLOSE);
    setIconImage( ImageIO.read(ClassLoader.getSystemResource("images/gold.gif")));
    setSize(600, 400);
    setTitle(bundle.getString("programTitle"));
    
    backgroundPanel = new BackgroundPanel();
    String bg = Preferences.userNodeForPackage(Main.class).get("backgroundImage", "null");
    if(!bg.equals("null")) backgroundPanel.setBackground(new File(bg));
    else {
      backgroundPanel.setBackground(new Color(Preferences.userNodeForPackage(Main.class).getInt("backgroundColor", 0xc8ddf2)));
    }
    setContentPane(backgroundPanel);
    
    /****JOptionPane****/
    UIManager.put("OptionPane.cancelButtonText", bundle.getString("cancel"));
    UIManager.put("OptionPane.noButtonText", bundle.getString("no"));
    UIManager.put("OptionPane.okButtonText", bundle.getString("ok"));
    UIManager.put("OptionPane.yesButtonText", bundle.getString("yes"));
    
    /****Menu****/
    setJMenuBar(menuBar = new ProgramMenuBar(this));
    
    adminsDialog = new AdminsDialog(this);
    announceButton = new JButton(bundle.getString("announceButton"));
    autowriter = new Autowriter(this, coordinatePanel = new CoordinatePanel());
    clipboardLabel = new JLabel();
    manager = KeyboardFocusManager.getCurrentKeyboardFocusManager();
    playersDialog = new PlayersDialog(this);
    clipboardLabel.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 5));
    
    JPopupMenu popupMenu = new JPopupMenu();
    fastAnnounceMenuItem = new JMenuItem(bundle.getString("fastAnnounceMenuItem"));
    lightningAnnounceMenuItem = new JMenuItem(bundle.getString("lightningAnnounceMenuItem"));
    popupMenu.add(fastAnnounceMenuItem);
    popupMenu.add(lightningAnnounceMenuItem);
    announceButton.addMouseListener(new PopupListener(popupMenu));
    fastAnnounceMenuItem.addActionListener(this);
    lightningAnnounceMenuItem.addActionListener(this);
    
    JPanel southPanel = new JPanel(new BorderLayout());
    southPanel.add(coordinatePanel, BorderLayout.WEST);
    southPanel.add(clipboardLabel, BorderLayout.CENTER);
    southPanel.add(announceButton, BorderLayout.EAST);
    southPanel.setBorder(BorderFactory.createEtchedBorder());
    
    setLayout(new BorderLayout());
    add(southPanel, BorderLayout.SOUTH);
    
    announceButton.addActionListener(this);
    manager.addKeyEventDispatcher(this);
    
    setEnabledResumeAutowrite(false);
  }
  
  public void newCupTournament() {
    if (!adminsDialog.showDialog()) {
      return;
    }
    
    if (playersDialog.showDialog(false)) {
      if(playersDialog.getPlayers().size() == 0) return;
      
      Cup cup = new Cup(this, playersDialog.getPlayers());
      
      JScrollPane scrollPane = new JScrollPane(cup);
      scrollPane.getVerticalScrollBar().setUnitIncrement(25);
      scrollPane.setOpaque(false);
      scrollPane.getViewport().setOpaque(false);
      
      add(scrollPane, BorderLayout.CENTER);
      
      tournament = cup;
      menuBar.setEnabledNewTournament(false);
      validate();
      Dimension prefSize = getPreferredSize();
      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      Dimension newSize = new Dimension(Math.min(prefSize.width, screenSize.width), Math.min(prefSize.height, screenSize.height));
      setSize(newSize);
      setLocationRelativeTo(null);
      validate();
      update();
    }
  }
  
  public void newDivisionTournament() {
    if (!adminsDialog.showDialog()) {
      return;
    }
    
    if (playersDialog.showDialog(true)) {
      League league = new League(this);
      league.makeDivisions(playersDialog.getPlayers(),
              playersDialog.getDivisionCount(),
              playersDialog.getGamesCount(), true);
      
      
      if (!league.isEmpty()) {
        menuBar.setEnabledNewTournament(false);
        add(league, BorderLayout.CENTER);
        tournament = league;
        validate();
        update();
      }
    }
  }
  
  public void nextRound() {
    if(!(tournament instanceof League)) return;
    
    League league = (League) tournament;
    
    ArrayList<String> list = league.getWinners();
    if (list.size() == 0) {
      return;
    }
    
    if (!league.isReady()) {
      int sel = JOptionPane.showConfirmDialog(this,
              bundle. getString("divisionsNotReadyMessage"),
              bundle.getString("divisionsNotReadyTitle"),
              JOptionPane.YES_NO_OPTION);
      
      if (sel != JOptionPane.YES_OPTION) {
        return;
      }
    }
    
    playersDialog.setPlayers(list);
    playersDialog.setDivisionCount(1);
    playersDialog.setGamesCount(6);
    if (playersDialog.showDialog(true)) {
      league.makeDivisions(playersDialog.getPlayers(),
              playersDialog.getDivisionCount(),
              playersDialog.getGamesCount(),
              false);
    }
    update();
  }
  
  private ArrayList<String> parseFile(String path) {
    ArrayList<String> lines = new ArrayList<String>();
    
    String admin1 = adminsDialog.getAdmin1();
    String admin2 = adminsDialog.getAdmin2();
    String prize1 = adminsDialog.getPrize1();
    String prize2 = adminsDialog.getPrize2();
    
    try {
      BufferedReader br = new BufferedReader(new FileReader(path));
      String line;
      while ( (line = br.readLine()) != null) {
        line = line.replaceAll("<admin1>", admin1);
        line = line.replaceAll("<admin2>", admin2);
        line = line.replaceAll("<first_prize>", prize1);
        line = line.replaceAll("<second_prize>", prize2);
        line = line.replaceAll("<gold>", gold);
        line = line.replaceAll("<silver>", silver);
        lines.add(line);
      }
      
      br.close();
    } catch (IOException exc) {}
    
    return lines;
  }
  
  public void processWindowEvent(WindowEvent e) {
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      if (tournament == null || tournament.isEmpty()) {
        System.exit(0);
      } else {
        JOptionPane.showMessageDialog(this,
                bundle.getString("closeWindowMessage"));
      }
    }
    
    if(e.getID() == WindowEvent.WINDOW_ACTIVATED) {
      if(autowriter != null && autowriter.isRunning()) {
        clipboardLabel.setText(bundle.getString("autowriteCancelled"));
        autowriter.stop();
      }
    }
    
    repaint();
  }
  
  public void resumeAutowrite() {
    autowriter.write();
  }
  
  public void setBackgroundColor(Color c) {
    backgroundPanel.setBackground(c);
    Preferences.userNodeForPackage(Main.class).putInt("backgroundColor", c.getRGB());
    Preferences.userNodeForPackage(Main.class).put("backgroundImage", "null");
  }
  
  public void setBackgroundImage(File file) {
    backgroundPanel.setBackground(file);
    Preferences.userNodeForPackage(Main.class).put("backgroundImage", file.getAbsolutePath());
  }
  
  public void setClipboardText(String str) {
    autowriter.setClipboardContents(str);
    clipboardLabel.setText(str);
  }
  
  public void setEnabledResumeAutowrite(boolean b) {
    menuBar.setEnabledResumeAutowrite(b);
  }
  
  public void tellRules() {
    Object obj = adminsDialog.getRulesFile();
    if(obj != null) autowriter.write(parseFile(obj.toString()));
  }
  
  public void thankPlayers() {
    String goldMed = tournament.getGold();
    String silverMed = tournament.getSilver();
    
    if (goldMed == null || silverMed == null) {
      JOptionPane.showMessageDialog(this,
              bundle.getString("selectGoldAndSilver"));
      return;
    }
    
    gold = goldMed;
    silver = silverMed;
    
    Object obj = adminsDialog.getThankFile();
    if(obj != null) autowriter.write(parseFile(obj.toString()));
  }
  
  public void update() {
    int size = 0;
    boolean b = false;
    
    if(tournament != null) {
      size = tournament.getUnannounced(false).size();
      b = size != 0;
    }
    
    announceButton.setText(bundle.getString("announceButton") + " (" + size + ")");
    announceButton.setEnabled(b);
    fastAnnounceMenuItem.setEnabled(b);
    lightningAnnounceMenuItem.setEnabled(b);
    if(tournament == null) return;
    menuBar.setEnabledNextRound(tournament instanceof League && ((League) tournament).getWinners().size() > 0);
    menuBar.setEnabledTellRules(!tournament.isEmpty());
    menuBar.setEnabledThankPlayers(tournament.isFinished());
  }
}
