package gui;

import java.io.*;
import java.net.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import javax.swing.text.Document;
import java.util.Locale;
import java.util.ResourceBundle;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class HelpFrame
    extends JFrame implements ActionListener, HyperlinkListener {

  private JEditorPane editorPane;
  private JMenuItem closeMenuItem;

  public HelpFrame() {
    ResourceBundle bundle = ResourceBundle.getBundle("GUI");
    setTitle(bundle.getString("helpFrameTitle"));
    setSize(new Dimension(500, 500));

    JMenuBar menuBar = new JMenuBar();
    JMenu fileMenu = new JMenu(bundle.getString("fileMenu"));
    closeMenuItem = new JMenuItem(bundle.getString("closeMenu"));

    setJMenuBar(menuBar);
    menuBar.add(fileMenu);
    fileMenu.add(closeMenuItem);
    closeMenuItem.addActionListener(this);

    String language = Locale.getDefault().getLanguage();

    try {
      String path;
      if (language.equals("da")) {
        path = "docs/dan_help.html";
      }
      else if (language.equals("sv")) {
        path = "docs/swe_help.html";
      }
      else {
        path = "docs/eng_help.html";
      }

      File f = new File(path);
      String s = f.getAbsolutePath();
      s = "file:" + s;

      editorPane = new JEditorPane(s);
      editorPane.setEditable(false);
      editorPane.addHyperlinkListener(this);

      setLayout(new BorderLayout());
      add(new JScrollPane(editorPane), BorderLayout.CENTER);
    }
    catch (IOException exc) {
      exc.printStackTrace();
    }

    setLocationRelativeTo(null);
    setVisible(true);
  }

  public void actionPerformed(ActionEvent e) {
    if(e.getSource() == closeMenuItem) {
      dispose();
    }
  }

  public void hyperlinkUpdate(HyperlinkEvent e) {
    if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
      linkActivated(e.getURL());
    }
  }

  protected void linkActivated(URL u) {
    Cursor c = editorPane.getCursor();
    Cursor waitCursor = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
    editorPane.setCursor(waitCursor);
    SwingUtilities.invokeLater(new PageLoader(u, c));
  }

  class PageLoader
      implements Runnable {

    PageLoader(URL u, Cursor c) {
      url = u;
      cursor = c;
    }

    public void run() {
      if (url == null) {
        // restore the original cursor
        editorPane.setCursor(cursor);

        // PENDING(prinz) remove this hack when
        // automatic validation is activated.
        Container parent = editorPane.getParent();
        parent.repaint();
      }
      else {
        Document doc = editorPane.getDocument();
        try {
          editorPane.setPage(url);
        }
        catch (IOException exc) {
          editorPane.setDocument(doc);
          exc.printStackTrace();
        }
        finally {
          // schedule the cursor to revert after
          // the paint has happended.
          url = null;
          SwingUtilities.invokeLater(this);
        }
      }
    }

    URL url;
    Cursor cursor;
  }
}
