package gui;

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class PlayersDialog
        extends JDialog implements ActionListener, DocumentListener {
  
  private String duplicates, playerCount;
  
  private JButton cancelButton, okButton;
  private JLabel countLabel, divCountLabel, gamesCountLabel;
  private JSpinner divCountSpinner, gamesCountSpinner;
  private JTextArea textArea;
  private SpinnerNumberModel divNumberModel, gamesNumberModel;
  
  private boolean accepted = false;
  
  public PlayersDialog(Frame frame) {
    super(frame, true);
    
    initComponents();
    
    setSize(400, 400);
    setLocationRelativeTo(null);
  }
  
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
    if (src == okButton) {
      accepted = true;
      setVisible(false);
    } else if (src == cancelButton) {
      accepted = false;
      setVisible(false);
    }
  }
  
  public void changedUpdate(DocumentEvent e) {
    updateCount();
  }
  
  private void createLayout(boolean extraOptions) {
  getContentPane().removeAll();
    
    JPanel divPanel = new JPanel(new FlowLayout());
    divPanel.add(divCountLabel);
    divPanel.add(divCountSpinner);
    
    JPanel gamesPanel = new JPanel(new FlowLayout());
    gamesPanel.add(gamesCountLabel);
    gamesPanel.add(gamesCountSpinner);
    
    JPanel buttonPanel = new JPanel(new FlowLayout());
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);
    
    JPanel northPanel = new JPanel(new BorderLayout());
    northPanel.add(divPanel, BorderLayout.WEST);
    northPanel.add(gamesPanel, BorderLayout.EAST);
    northPanel.setBorder(BorderFactory.createRaisedBevelBorder());
    
    JPanel southPanel = new JPanel(new BorderLayout());
    southPanel.add(countLabel, BorderLayout.WEST);
    southPanel.add(buttonPanel, BorderLayout.EAST);
    southPanel.setBorder(BorderFactory.createRaisedBevelBorder());
    
    setLayout(new BorderLayout());
    if(extraOptions) add(northPanel, BorderLayout.NORTH);
    add(new JScrollPane(textArea), BorderLayout.CENTER);
    add(southPanel, BorderLayout.SOUTH);
  }
  
  private int getCount() {
    ArrayList<String> temp = new ArrayList<String> ();
    
    try {
      BufferedReader br = new BufferedReader(new StringReader(textArea.getText()));
      String line;
      
      while ( (line = br.readLine()) != null) {
        if ( (line = line.trim()).length() > 0) {
          if (temp.contains(line)) {
            br.close();
            return -1;
          } else {
            temp.add(line);
          }
        }
      }
    } catch (IOException exc) {
      exc.printStackTrace();
    }
    
    return temp.size();
  }
  
  public int getDivisionCount() {
    return (Integer) divNumberModel.getValue();
  }
  
  public int getGamesCount() {
    return (Integer) gamesNumberModel.getValue();
  }
  
  public ArrayList<String> getPlayers() {
    ArrayList<String> players = new ArrayList<String> ();
    
    try {
      BufferedReader br = new BufferedReader(new StringReader(textArea.getText()));
      String line;
      
      while ( (line = br.readLine()) != null) {
        if ( (line = line.trim()).length() > 0) {
          players.add(line);
        }
      }
      br.close();
    } catch (IOException exc) {
      exc.printStackTrace();
    }
    
    return players;
  }
  
  private void initComponents() {
    ResourceBundle bundle = ResourceBundle.getBundle("GUI");
    setTitle(bundle.getString("playersDialogTitle"));
    
    duplicates = bundle.getString("containsDuplicates");
    playerCount = bundle.getString("playerCount");
    
    cancelButton = new JButton(bundle.getString("cancel"));
    cancelButton.addActionListener(this);
    countLabel = new JLabel();
    countLabel.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 0));
    divCountSpinner = new JSpinner(divNumberModel = new SpinnerNumberModel(4, 1,
            Integer.MAX_VALUE, 1));
    divCountSpinner.setPreferredSize(new Dimension(50, 20));
    divCountLabel = new JLabel(bundle.getString("selectDivisionCount"));
    divCountLabel.setLabelFor(divCountSpinner);
    
    gamesCountSpinner = new JSpinner(gamesNumberModel =
            new SpinnerNumberModel(6, 1,
            Integer.MAX_VALUE, 1));
    gamesCountSpinner.setPreferredSize(new Dimension(50, 20));
    gamesCountLabel = new JLabel(bundle.getString("selectGameCount"));
    gamesCountLabel.setLabelFor(gamesCountSpinner);
    
    okButton = new JButton(bundle.getString("ok"));
    okButton.addActionListener(this);
    textArea = new JTextArea();
    textArea.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    textArea.setFont(new Font("Dialog", Font.BOLD, 12));
    textArea.getDocument().addDocumentListener(this);
  }
  
  public void insertUpdate(DocumentEvent e) {
    updateCount();
  }
  
  public void setDivisionCount(int c) {
    divCountSpinner.setValue(c);
  }
  
  public void setGamesCount(int c) {
    gamesCountSpinner.setValue(c);
  }
  
  public void setPlayers(ArrayList<String> list) {
    StringBuilder bui = new StringBuilder();
    
    for (String str : list) {
      bui.append(str + '\n');
    }
    
    textArea.setText(bui.toString());
  }
  
  public void removeUpdate(DocumentEvent e) {
    updateCount();
  }
  
  public boolean showDialog(boolean extraOptions) {
    accepted = false;
    createLayout(extraOptions);
    setVisible(true);
    return accepted;
  }
  
  private void updateCount() {
    int c = getCount();
    
    if (c == -1) {
      countLabel.setForeground(Color.red);
      countLabel.setText(duplicates);
      okButton.setEnabled(false);
    } else {
      countLabel.setForeground(Color.black);
      countLabel.setText(playerCount + ": " + c);
      okButton.setEnabled(true);
    }
  }
}
