package gui;

import divisionizer.Main;
import java.awt.*;
import java.util.ResourceBundle;
import java.awt.event.*;
import java.io.*;
import java.util.prefs.Preferences;
import javax.swing.*;

public class ProgramMenuBar extends JMenuBar implements ActionListener {
  
  private JMenu actionMenu,backgroundMenu, closeMenu, exitMenu, fileMenu, helpMenu,
          languageMenu,miscMenu,newTournamentMenu, settingsMenu;
  private JMenuItem aboutMenuItem, colorMenuItem, cupMenuItem, danishMenuItem, divisionMenuItem, englishMenuItem, finnishMenuItem, helpMenuItem,
          imageMenuItem, nextRoundMenuItem, nowMenuItem, resumeAutowriteMenuItem, swedishMenuItem,
          tellRulesMenuItem, thankPlayersMenuItem;
  private JCheckBoxMenuItem checkListCheckBoxMenuItem;
  private JFileChooser fileChooser;
  
  private HeadFrame headFrame;
  
  public ProgramMenuBar(HeadFrame _headFrame) {
    headFrame = _headFrame;
    
    initComponents();
  }
  
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
    
    if (src == aboutMenuItem) {
      new AboutFrame();
    } else if(src == checkListCheckBoxMenuItem) {
      Preferences.userNodeForPackage(divisionizer.Main.class).putBoolean("checkList", checkListCheckBoxMenuItem.isSelected());
    }else if (src == colorMenuItem) {
      Color c = (new JColorChooser()).showDialog(headFrame, "", new Color(0xc8ddf2));
      if(c!=null) headFrame.setBackgroundColor(c);
    }else if (src == cupMenuItem) {
      headFrame.newCupTournament();
    } else if (src == danishMenuItem) {
      setLanguage("da", "DK");
    } else if (src == englishMenuItem) {
      setLanguage("en", "GB");
    }  else if (src == finnishMenuItem) {
      setLanguage("fi", "FI");
    } else if(src == helpMenuItem) {
      new HelpFrame();
    } else if (src == divisionMenuItem) {
      headFrame.newDivisionTournament();
    } else if(src == imageMenuItem) {
      fileChooser.showOpenDialog(headFrame);
      File file = fileChooser.getSelectedFile();
      if(file != null) headFrame.setBackgroundImage(file);
    } else if (src == nextRoundMenuItem) {
      headFrame.nextRound();
    } else if (src == nowMenuItem) {
      headFrame.closeProgram();
    } else if(src == resumeAutowriteMenuItem) {
      headFrame.resumeAutowrite();
    } else if (src == swedishMenuItem) {
      setLanguage("sv", "SE");
    } else if (src == tellRulesMenuItem) {
      headFrame.tellRules();
    } else if (src == thankPlayersMenuItem) {
      headFrame.thankPlayers();
    }
  }
  
  private void initComponents() {
    ResourceBundle bundle = ResourceBundle.getBundle("GUI");
    
    fileChooser = new JFileChooser();
    fileChooser.setFileFilter(new javax.swing.filechooser.FileFilter() {
      public boolean accept(File file) {
        String name = file.getName().toLowerCase();
        return file.isDirectory() || name.endsWith(".jpg") || name.endsWith(".jpeg") || name.endsWith(".gif") || name.endsWith(".png");
      }
      
      public String getDescription() {
        return "Image Files (*.jpg,*.jpeg,*.gif,*.png)";
      }
    });
    
    aboutMenuItem = new JMenuItem(bundle.getString("aboutMenuItem"));
    actionMenu = new JMenu(bundle.getString("actionMenu"));
    backgroundMenu = new JMenu(bundle.getString("backgroundMenu"));
    checkListCheckBoxMenuItem = new JCheckBoxMenuItem(bundle.getString("checkListCheckBoxMenuItem"), Preferences.userNodeForPackage(divisionizer.Main.class).getBoolean("checkList", false));
    closeMenu = new JMenu(bundle.getString("closeMenu"));
    colorMenuItem = new JMenuItem(bundle.getString("colorMenuItem"));
    cupMenuItem = new JMenuItem(bundle.getString("cupMenuItem"));
    danishMenuItem = new JMenuItem(new ImageIcon(ClassLoader.getSystemResource("images/flag-dk.gif")));
    divisionMenuItem = new JMenuItem(bundle.getString("divisionMenuItem"));
    englishMenuItem = new JMenuItem(new ImageIcon(ClassLoader.getSystemResource("images/flag-uk.gif")));
    exitMenu = new JMenu(bundle.getString("exitMenu"));
    fileMenu = new JMenu(bundle.getString("fileMenu"));
    finnishMenuItem = new JMenuItem(new ImageIcon(ClassLoader.getSystemResource("images/flag-fi.gif")));
    helpMenu = new JMenu(bundle.getString("helpMenu"));
    helpMenuItem = new JMenuItem(bundle.getString("helpMenuItem"));
    imageMenuItem = new JMenuItem(bundle.getString("imageMenuItem"));
    languageMenu = new JMenu(bundle.getString("languageMenu"));
    miscMenu = new JMenu(bundle.getString("miscMenu"));
    nextRoundMenuItem = new JMenuItem(bundle.getString("nextRoundMenuItem"));
    newTournamentMenu = new JMenu(bundle.getString("newTournamentMenu"));
    nowMenuItem = new JMenuItem(bundle.getString("nowMenuItem"));
    resumeAutowriteMenuItem = new JMenuItem(bundle.getString("resumeAutowriteMenuItem"));
    settingsMenu = new JMenu(bundle.getString("settingsMenu"));
    swedishMenuItem = new JMenuItem(new ImageIcon(ClassLoader.getSystemResource("images/flag-se.gif")));
    tellRulesMenuItem = new JMenuItem(bundle.getString("tellRulesMenuItem"));
    thankPlayersMenuItem = new JMenuItem(bundle.getString(
            "thankPlayersMenuItem"));
    
    nextRoundMenuItem.setEnabled(false);
    tellRulesMenuItem.setEnabled(false);
    thankPlayersMenuItem.setEnabled(false);
    
    add(fileMenu);
    fileMenu.add(newTournamentMenu);
    newTournamentMenu.add(cupMenuItem);
    newTournamentMenu.add(divisionMenuItem);
    fileMenu.add(closeMenu);
    closeMenu.add(exitMenu);
    exitMenu.add(nowMenuItem);
    add(actionMenu);
    actionMenu.add(resumeAutowriteMenuItem);
    actionMenu.addSeparator();
    actionMenu.add(nextRoundMenuItem);
    actionMenu.add(tellRulesMenuItem);
    actionMenu.add(thankPlayersMenuItem);
    add(settingsMenu);
    settingsMenu.add(languageMenu);
    languageMenu.add(englishMenuItem);
    languageMenu.add(danishMenuItem);
    languageMenu.add(finnishMenuItem);
    languageMenu.add(swedishMenuItem);
    settingsMenu.add(backgroundMenu);
    backgroundMenu.add(imageMenuItem);
    backgroundMenu.add(colorMenuItem);
    settingsMenu.add(miscMenu);
    miscMenu.add(checkListCheckBoxMenuItem);
    add(helpMenu);
    helpMenu.add(helpMenuItem);
    helpMenu.add(aboutMenuItem);
    
    aboutMenuItem.addActionListener(this);
    colorMenuItem.addActionListener(this);
    checkListCheckBoxMenuItem.addActionListener(this);
    cupMenuItem.addActionListener(this);
    danishMenuItem.addActionListener(this);
    divisionMenuItem.addActionListener(this);
    englishMenuItem.addActionListener(this);
    finnishMenuItem.addActionListener(this);
    helpMenuItem.addActionListener(this);
    imageMenuItem.addActionListener(this);
    nextRoundMenuItem.addActionListener(this);
    nowMenuItem.addActionListener(this);
    resumeAutowriteMenuItem.addActionListener(this);
    swedishMenuItem.addActionListener(this);
    tellRulesMenuItem.addActionListener(this);
    thankPlayersMenuItem.addActionListener(this);
  }
  
  public void setEnabledNewTournament(boolean b) {
    newTournamentMenu.setEnabled(b);
  }
  
  public void setEnabledNextRound(boolean b) {
    nextRoundMenuItem.setEnabled(b);
  }
  
  public void setEnabledResumeAutowrite(boolean b) {
    resumeAutowriteMenuItem.setEnabled(b);
  }
  
  public void setEnabledTellRules(boolean b) {
    tellRulesMenuItem.setEnabled(b);
  }
  
  public void setEnabledThankPlayers(boolean b) {
    thankPlayersMenuItem.setEnabled(b);
  }
  
  private void setLanguage(String language, String country) {
    Preferences.userNodeForPackage(Main.class).put("language", language);
    Preferences.userNodeForPackage(Main.class).put("country", country);
  }
}
