package tournament.cup;

import gui.HeadFrame;
import java.awt.*;
import java.util.ArrayList;
import java.util.Random;
import java.io.*;
import javax.swing.*;
import tournament.*;

public class Cup extends JPanel implements Serializable, Tournament {
  
  private final Font font = new Font("Dialog",Font.BOLD, 12);
  
  private ArrayList<String> players;
  private ArrayList<CupField> cupFields = new ArrayList<CupField>();
  private HeadFrame headFrame;
  
  private int tournamentSize = 0;
  private int columns = 1;
  
  Random random = new Random();
  
  public Cup(HeadFrame _headFrame, ArrayList<String> _players) {
    setOpaque(false);
    
    headFrame = _headFrame;
    players = _players;
    shufflePlayers();
    
    setBackground(Color.white);
    setFont(font);
    setLayout(null);
    
    addFields(players);
    
  }
  
  private void addColumn(int number) {
    int height = tournamentSize >> number + 1;
    
    Dimension size = CupField.PREFERRED_SIZE;
    Insets insets = getInsets();
    
    int paneHeight = (size.height + 4)*(tournamentSize >> 1);
    
    int x = insets.left + 25 + ((int) size.getWidth() + 10)*number;
    int y = 6 + (paneHeight / height)/2 - (paneHeight / (tournamentSize >> 1)) / 2;
    
    for(int i = 0; i < height; i++) {
      int mods = 0;
      
      if(number == columns - 1) {
        mods |= CupField.FINAL;
      } else if(number == columns - 2) {
        mods |= CupField.SEMI;
      } else if(number == columns - 3) {
        mods |= CupField.QUARTER;
      } else if(number == columns - 4) {
        mods |= CupField.SEMIQUARTER;
      } else {
        mods |= CupField.NORMAL;
      }
      
      if(number == 0) mods |= CupField.FIRST_ROUND;
      
      CupField field = new CupField(this, mods, cupFields.size() + 1, random.nextBoolean());
      cupFields.add(field);
      add(field);
      
      field.setBounds(x ,insets.top + y,size.width, size.height);
      y+= (paneHeight / height);
    }
    
    if(number == 0) {
      Dimension s = new Dimension();
      s.height = insets.bottom + insets.top + paneHeight + 6;
      s.width = insets.left + 50 + ((int) size.getWidth() + 10)*columns;
      
      setPreferredSize(s);
      validate();
    }
  }
  
  private void addFields(ArrayList<String> players) {
    for(tournamentSize = 2; tournamentSize < players.size(); tournamentSize<<=1) {
      columns++;
    }
    
    for(int i = 0; (1 << i) < tournamentSize; i++) {
      addColumn(i);
    }
    
    for(int i = 0; i < players.size(); i++) {
      cupFields.get(i % (tournamentSize >> 1)).setPlayer(players.get(i), 2*i >= tournamentSize);
    }
  }
  
  private void checkColumn(int column) {
    int fo = getFirstOrdinal(column);
    int preFo = getFirstOrdinal(column - 1);
    int ch = getColumnHeight(column);
    
    for(int i = 0; i < ch; i++) {
      CupField field = cupFields.get(fo + i);
      
      for(int j = 0; j < 2; j++) {
        CupField pre = cupFields.get(preFo + (i*2) + j);
        if(pre.isFinished()) {
          field.setPlayer(pre.getWinner(), (j & 1) == 0);
        }
      }
    }
  }
  
  public  boolean containsName(String name) {
    for(CupField c : cupFields) {
      if(c.isFirstRound() && c.hasPlayer(name)) return true;
    }
    return false;
  }
  
  private int getColumnHeight(int column) {
    return tournamentSize >> (column + 1);
  }
  
  private int getFirstOrdinal(int column) {
    int c = 0;
    
    for(int i = 0; i < column; i++) {
      c += tournamentSize >> (i + 1);
    }
    
    return c;
  }
  
  public String getGold() {
    return cupFields.get(getFirstOrdinal(columns - 1)).getWinner();
  }
  public String getSilver() {
    return cupFields.get(getFirstOrdinal(columns - 1)).getLoser();
  }
  public ArrayList<String> getUnannounced(boolean mark) {
    ArrayList<String> temp = new ArrayList<String>();
    
    for(CupField c : cupFields) {
      int i = 0;
      i+=c.isEmpty(true)?1:0;
      i+=c.isEmpty(false)?1:0;
      
      if(c.isUnannounced()) {
        if(c.isFreeRound() && i == 1) {
          temp.add(c.getName());
          if(mark) c.setWinner(c.isEmpty(false));
        } else if(i == 0) {
          temp.add(c.getName());
          if(mark) c.setAnnounced(true);
        }
      }
    }
    if(mark) update();
    return temp;
  }

  public boolean isEmpty() {
    return columns == 0;
  }
  public boolean isFinished() {
    return cupFields.get(getFirstOrdinal(columns - 1)).isFinished();
  }

  public void paintComponent(Graphics g) {
    super.paintComponent(g);
  }
  
  public void setClipboardText(String str) {
    headFrame.setClipboardText(str);
  }
  
  public void shufflePlayers() {
    ArrayList<String> temp = new ArrayList<String>();
    
    while(players.size() > 0) {
      int i = random.nextInt(players.size());
      temp.add(players.get(i));
      players.remove(i);
    }
    
    players = temp;
  }
  
  public void update() {
    for(CupField c : cupFields) {
      c.update();
    }
    
    for(int i = 1; i < columns; i++) {
      checkColumn(i);
    }
    
    headFrame.update();
  }
}
