package tournament.div;

import java.util.*;

import java.awt.*;
import javax.swing.*;

public class Division
        extends JPanel {
  
  private String goldText, silverText, winnerText, winnerByLotteryText;
  
  private JPanel playersPanel, matchesPanel;
  private JTextArea textArea;
  
  public String divisionID;
  private int gamesCount;
  private boolean finalDivision;
  private League league;
  private ArrayList<Match> matches = new ArrayList<Match> ();
  private ArrayList<Player> players = new ArrayList<Player> ();
  
  public Division(League _league, String _divisionID, int _gamesCount,
          boolean _finalDivision) {
    
    league = _league;
    divisionID = _divisionID;
    gamesCount = _gamesCount;
    finalDivision = _finalDivision;
    
    initComponents();
  }
  
  public void addPlayer(Player p) {
    players.add(p);
    playersPanel.add(p);
  }
  
  public boolean containsName(String name) {
    for(Player p : players) {
      if(p.getName().equals(name)) return true;
    }
    return false;
  }
  
  public boolean findFreeMatch() {
    int size = players.size();
    
    //Try to find an opponent for the player who has played least games
    for (int i = 0; i < size - 1; i++) {
      for (int j = 0; j < size; j++) {
        Player p1 = players.get(j);
        if (!p1.isPlaying() && !p1.isDisqualified() && p1.getGames() == i ) {
          for (int k = 0; k < size; k++) {
            if (k == j) {
              continue;
            }
            Player p2 = players.get(k);
            if (!p2.isPlaying() && !p1.hasPlayed(p2) && !p2.isDisqualified()) {
              Match m = new Match(divisionID + (matches.size() + 1),
                      p1, p2, gamesCount);
              matchesPanel.add(m);
              matchesPanel.revalidate();
              matches.add(m);
              return true;
            }
          }
        }
      }
    }
    return false;
  }
  
  public void findFreeMatches() {
    while (findFreeMatch());
  }
  
  public ArrayList<Player> findWinners(ArrayList<Player> players) {
    float max = 0f;
    ArrayList<Player> winners = new ArrayList<Player> ();
    
    for (Player p : players) {
      if (p.isDisqualified()) {
        continue;
      }
      
      float tp = p.getPoints();
      
      if (tp > max) {
        max = tp;
        winners.clear();
        winners.add(p);
      } else if (tp == max) {
        winners.add(p);
      }
    }
    
    if (winners.size() == 2) {
      Player p1 = winners.get(0);
      Player p2 = winners.get(1);
      
      for (Match m : matches) {
        if (m.hasPlayer(p1) && m.hasPlayer(p2)) {
          float po1 = m.getPointsFor(p1);
          float po2 = m.getPointsFor(p2);
          
          if (po1 > po2) {
            winners.clear();
            winners.add(p1);
          } else if (po2 > po1) {
            winners.clear();
            winners.add(p2);
          }
        }
      }
    }
    
    int leastLosses = Integer.MAX_VALUE;
    ArrayList<Player> temp = new ArrayList<Player> ();
    
    for (Player p : winners) {
      int losses = p.getLosses();
      
      if (losses < leastLosses) {
        leastLosses = losses;
        temp.clear();
        temp.add(p);
      } else if (leastLosses == losses) {
        temp.add(p);
      }
    }
    
    winners = temp;
    
    return winners;
  }
  
  public League getLeague() {
    return league;
  }
  
  public ArrayList<Match> getUnannounced() {
    ArrayList<Match> list = new ArrayList<Match> ();
    
    for (Match m : matches) {
      if (!m.isAnnounced()) {
        list.add(m);
      }
    }
    return list;
  }
  
  public ArrayList<Player> getGolds() {
    ArrayList<Player> list = new ArrayList<Player> ();
    
    for (Player p : players) {
      if (p.getMerit() == Player.Merit.GOLD) {
        list.add(p);
      }
    }
    
    return list;
  }
  
  public ArrayList<Player> getSilvers() {
    ArrayList<Player> list = new ArrayList<Player> ();
    
    for (Player p : players) {
      if (p.getMerit() == Player.Merit.SILVER) {
        list.add(p);
      }
    }
    
    return list;
  }
  
  public ArrayList<Player> getWinners() {
    ArrayList<Player> list = new ArrayList<Player> ();
    
    for (Player p : players) {
      if (p.getMerit() == Player.Merit.WINNER) {
        list.add(p);
      }
    }
    
    return list;
  }
  
  public boolean hasUnannounced() {
    for (Match m : matches) {
      if (!m.isAnnounced()) {
        return true;
      }
    }
    return false;
  }
  
  private void initComponents() {
    setOpaque(false);
    
    ResourceBundle bundle = ResourceBundle.getBundle("Division");
    
    goldText = bundle.getString("gold");
    silverText = bundle.getString("silver");
    winnerText = bundle.getString("winner");
    winnerByLotteryText = bundle.getString("winnerByLottery");
    
    playersPanel = new JPanel(new GridLayout(0, 1));
    playersPanel.setOpaque(false);
    matchesPanel = new JPanel(new GridLayout(0, 1));
    matchesPanel.setOpaque(false);
    
    textArea = new JTextArea();
    textArea.setBorder(BorderFactory.createLoweredBevelBorder());
    textArea.setEditable(false);
    textArea.setFont(new Font("Dialog", Font.BOLD, 12));
    textArea.setWrapStyleWord(true);
    textArea.setLineWrap(true);
    
    JPanel westPanel = new JPanel(new BorderLayout());
    westPanel.add(playersPanel, BorderLayout.NORTH);
    westPanel.setOpaque(false);
    
    JPanel eastPanel = new JPanel(new BorderLayout());
    eastPanel.add(matchesPanel, BorderLayout.NORTH);
    eastPanel.setOpaque(false);
    
    JScrollPane westScrollPane = new JScrollPane(westPanel);
    westScrollPane.setOpaque(false);
    westScrollPane.getVerticalScrollBar().setUnitIncrement(15);
    westScrollPane.getViewport().setOpaque(false);
   
    JScrollPane eastScrollPane = new JScrollPane(eastPanel);
    eastScrollPane.setOpaque(false);
    eastScrollPane.getVerticalScrollBar().setUnitIncrement(15);
    eastScrollPane.getViewport().setOpaque(false);
    
    JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
            westScrollPane, eastScrollPane);
    splitPane.setOpaque(false);
    
    setLayout(new BorderLayout());
    add(splitPane, BorderLayout.CENTER);
    add(textArea, BorderLayout.SOUTH);
//    splitPane.setDividerLocation(0.5f);
  }
  
  public boolean isFinal() {
    return finalDivision;
  }
  
  public boolean isReady() {
    int activePlayers = 0;
    
    for (Player p : players) {
      if (!p.isDisqualified()) {
        activePlayers++;
      }
    }
    
    for (Player p : players) {
      if (!p.isDisqualified() && p.getGames() < activePlayers - 1) {
        return false;
      }
    }
    
    return true;
  }
  
  public void setEnabled(boolean b) {
    super.setEnabled(b);
    
    for (Player p : players) {
      p.setEnabled(b);
    }
    
    for (Match m : matches) {
      m.setEnabled(b);
      
      if (!b) {
        m.setAnnounced(true);
      }
    }
    
    update();
  }
  
  public void update() {
    for (Player p : players) {
      p.update();
    }
    
    for (Match m : matches) {
      m.update();
    }
    
    if (isReady()) {
      ArrayList<Player> winners = findWinners(players);
      String str = (finalDivision ? goldText : winnerText) + ": ";
      Player winner = null;
      
      if (winners.size() == 1) {
        winner = winners.get(0);
        str += winner.getName();
      } else if (winners.size() > 1) {
        
        for (Player p : winners) {
          str += p.getName() + ", ";
        }
        
        winner = winners.get(0);
        for (Player p : winners) {
          if (p.getLotteryChances() < winner.getLotteryChances()) {
            winner = p;
          }
        }
        
        str += "\n" + winnerByLotteryText + ": " + winner.getName();
      }
      
      if (finalDivision && winner != null) {
        winners = (ArrayList<Player>) players.clone();
        winners.remove(winner);
        winners = findWinners(winners);
        
        str += '\n' + silverText + ": ";
        
        if (winners.size() == 1) {
          winner = winners.get(0);
          str += winner.getName();
        } else if (winners.size() > 1) {
          
          for (Player p : winners) {
            str += p.getName() + ", ";
          }
          
          winner = winners.get(0);
          for (Player p : winners) {
            if (p.getLotteryChances() < winner.getLotteryChances()) {
              winner = p;
            }
          }
          
          str += '\n' + winnerByLotteryText + ": " + winner.getName();
        }
      }
      
      textArea.setText(str);
    } else {
      textArea.setText("");
    }
    
    findFreeMatches();
  }
  
  public void updateAll() {
    league.update();
  }
}
