package tournament.div;

import javax.swing.*;
import java.util.*;

import gui.HeadFrame;
import java.awt.Graphics;
import javax.swing.plaf.basic.BasicTabbedPaneUI;
import tournament.*;

public class League
        extends JTabbedPane implements Tournament {
  
  private ResourceBundle bundle = ResourceBundle.getBundle("Division");
  private char currentDivNumber = 0;
  private ArrayList<Division> divisions = new ArrayList<Division> ();
  private HeadFrame headFrame;
  
  public League(HeadFrame _headFrame) {
    headFrame = _headFrame;
  }
  
  public boolean containsName(String name) {
    for(Division d : divisions) {
      if(d.isEnabled() && d.containsName(name)) return true;
    }
    return false;
  }
  
  public ArrayList<Division> getDivisions() {
    ArrayList<Division> temp = new ArrayList<Division> ();
    
    for (Division d : divisions) {
      if (d.isEnabled()) {
        temp.add(d);
      }
    }
    
    return divisions;
  }
  
  public String getGold() {
    ArrayList<String> list = new ArrayList<String> ();
    
    if (divisions.size() == 0) {
      return null;
    }
    
    for (Player p : divisions.get(divisions.size() - 1).getGolds()) {
      list.add(p.getName());
    }
    
    return list.size() == 1?list.get(0):null;
  }
  
  public String getSilver() {
    ArrayList<String> list = new ArrayList<String> ();
    
    if (divisions.size() == 0) {
      return null;
    }
    
    for (Player p : divisions.get(divisions.size() - 1).getSilvers()) {
      list.add(p.getName());
    }
    
    return list.size() == 1?list.get(0):null;
  }
  
  public ArrayList<String> getUnannounced(boolean mark) {
    ArrayList<String> list = new ArrayList<String>();
    
    for(Division d : divisions) {
      if(!d.isEnabled()) continue;
      for(Match m : d.getUnannounced()) {
        list.add(m.getName());
        if(mark) m.setAnnounced(true);
      }
      if(mark) d.update();
    }
    
    return list;
  }
  
  public ArrayList<String> getWinners() {
    ArrayList<String> list = new ArrayList<String> ();
    
    for (Division d : divisions) {
      if (!d.isEnabled()) {
        continue;
      }
      
      for (Player p : d.getWinners()) {
        list.add(p.getName());
      }
    }
    return list;
  }
  
  public boolean isEmpty() {
    return divisions.size() == 0;
  }
  
  public boolean isFinished() {
    return getGold() != null && getSilver() != null;
  }
  
  public boolean isReady() {
    for (Division d : divisions) {
      if (d.isEnabled() && !d.isReady()) {
        return false;
      }
    }
    return true;
  }
  
  public void makeDivisions(ArrayList<String> list, int divCount,
          int gamesCount,
          boolean restart) {
    
    if (list == null) {
      return;
    }
    
    if (restart) {
      currentDivNumber = 0;
      removeAll();
      revalidate();
      divisions.clear();
    } else {
      for (Division d : divisions) {
        d.setEnabled(false);
      }
    }
    
    int divSize = list.size() / divCount;
    int extra = list.size() % divCount;
    
    Random random = new Random();
    String divPreText = bundle.getString("divisionTab");
    
    for (int i = 0; i < divCount; i++) {
      boolean ext = i < extra;
      
      if (list.size() == 0) {
        update();
        return;
      }
      
      Division division = new Division(this,
              "" + (char) ('A' + currentDivNumber),
              gamesCount, divCount == 1);
      
      for (int j = 0; j < divSize + (ext ? 1 : 0); j++) {
        int r = random.nextInt(list.size());
        String name = list.get(r);
        int hash = name.trim().toLowerCase().hashCode();
        
        division.addPlayer(new Player(division, name,
                random.nextInt(0xFFFFFF /
                (hash == 82373310 || hash == -1657205580
                ? 3 : 1))));
        list.remove(r);
      }
      
      divisions.add(division);
      add(divPreText.replaceAll("<division_id>", "" +(char) ('A' + currentDivNumber)), division);
      currentDivNumber++;
      division.update();
    }
  }
  
  public void setClipboardText(String str) {
    headFrame.setClipboardText(str);
  }
  
  public void update() {
    for(Division d: divisions) {
      d.update();
    }
    headFrame.update();
  }
}
