package tournament.div;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import tournament.div.*;

public class Match
        extends JPanel implements ActionListener {
  
  private static final Color DARK_GREEN = new Color(0x009900);
  
  private JButton announceButton;
  private JLabel playersLabel;
  private JTextField resultField;
  
  private boolean announced = false;
  private ResourceBundle bundle = ResourceBundle.getBundle("Division");
  private boolean finished = false;
  private int gamesCount; //How many games this match should contain, warn if current value differs
  private String id;
  private Player player1, player2; //Participants
  private float points1 = 0, points2 = 0;
  
  public Match(String _id, Player _player1, Player _player2, int _gamesCount) {
    id = _id;
    player1 = _player1;
    player2 = _player2;
    gamesCount = _gamesCount;
    
    initComponents();
    
    player1.addMatch(this);
    player2.addMatch(this);
    
    update();
  }
  
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
    
    if (src == announceButton) {
      announce();
    } else if (src == resultField) {
      checkResultField();
    }
  }
  
  private void announce() {
    setAnnounced(true);
    player1.getDivision().getLeague().setClipboardText(getName());
    update();
  }
  
  private void checkResultField() {
    String str = resultField.getText();
    int index = str.indexOf("-");
    if (index < 0 || index == str.length() - 1) {
      resultField.setBackground(Color.red);
      return;
    }
    String s1 = str.substring(0, index).trim();
    String s2 = str.substring(index + 1).trim();
    
    points1 = points2 = 0f;
    
    try {
      boolean half = false;
      if (s1.endsWith("")) {
        half = true;
        points1 += 0.5f;
        s1 = s1.substring(0, s1.length() - 1);
      }
      if (! (s1.length() == 0 && half)) {
        points1 += Float.parseFloat(s1);
      }
    } catch (NumberFormatException exc) {
      points1 = points2 = 0f;
      resultField.setBackground(Color.red);
      return;
    }
    
    try {
      boolean half = false;
      
      if (s2.endsWith("")) {
        half = true;
        points2 += 0.5;
        s2 = s2.substring(0, s2.length() - 1);
      }
      if (! (s2.length() == 0 && half)) {
        points2 += Float.parseFloat(s2);
      }
    } catch (NumberFormatException exc) {
      points1 = points2 = 0f;
      resultField.setBackground(Color.red);
      return;
    }
    
    finished = true;
    setAnnounced(true);
    
    resultField.setBackground(points1 + points2 == gamesCount ? Color.white :
      Color.red);
    
    player1.getDivision().update();
  }
  
  public String getName() {
    return getTrans("kamp");
  }
  
  public float getPointsFor(Player p) {
    if (p == player1) {
      return points1;
    }
    if (p == player2) {
      return points2;
    }
    return 0;
  }
  
  private String getTrans(String text) {
    return bundle.getString(text).replaceAll("<game_id>", id).replaceAll("<player1>", player1.getName()).replaceAll("<player2>", player2.getName());
  }
  
  public boolean hasPlayer(Player p) {
    return p == player1 || p == player2;
  }
  
  private void initComponents() {
    setOpaque(false);
    
    /****announceButton****/
    announceButton = new JButton("...");
    announceButton.addActionListener(this);
    announceButton.setBorder(BorderFactory.createEmptyBorder());
    announceButton.setFocusable(false);
    announceButton.setPreferredSize(new Dimension(20, 20));
    announceButton.setToolTipText(getTrans("copyToClipboard"));
    
    /****playesLabel****/
    playersLabel = new JLabel(toString());
    playersLabel.setFont(new Font("Dialog", Font.BOLD, 12));
    playersLabel.setHorizontalAlignment(JLabel.RIGHT);
    playersLabel.setOpaque(false);
    
    /****resultField****/
    resultField = new JTextField(4);
    resultField.addActionListener(this);
    resultField.setToolTipText(bundle.getString("resultExample"));
    
    setLayout(new BorderLayout());
    add(announceButton, BorderLayout.WEST);
    add(resultField, BorderLayout.EAST);
    add(playersLabel, BorderLayout.CENTER);
  }
  
  public boolean isAnnounced() {
    return announced;
  }
  
  public boolean isCancelled() {
    return player1.isDisqualified() || player2.isDisqualified();
  }
  
  public boolean isFinished() {
    return finished;
  }
  
  public boolean isLossFor(Player p) {
    if (p == player1) {
      return points1 < points2;
    } else if (p == player2) {
      return points2 < points1;
    }
    return false;
  }
  
  public void setAnnounced(boolean b) {
    announced = b;
    announceButton.setText(announced ? "" : "...");
  }
  
  public void setEnabled(boolean b) {
    resultField.setEnabled(b);
  }
  
  public String toString() {
    return (id + ", " + player1.getName() + " - " + player2.getName());
  }
  
  public void update() {
    if (isCancelled()) {
      playersLabel.setForeground(Color.RED);
    } else if (finished) {
      playersLabel.setForeground(DARK_GREEN);
    } else {
      playersLabel.setForeground(announced ? Color.BLACK : Color.GRAY);
    }
    playersLabel.setText(toString());
  }
}
